"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpTracingService = void 0;
const axios_1 = require("axios");
const nest_stack_context_1 = require("@zcong/nest-stack-context");
const rxjs_1 = require("rxjs");
const common_1 = require("@nestjs/common");
const opentracing_1 = require("opentracing");
const http_constants_1 = require("./http.constants");
const nest_stack_tracing_1 = require("@zcong/nest-stack-tracing");
let HttpTracingService = class HttpTracingService extends common_1.HttpService {
    constructor(config, instance, tracer) {
        if (config) {
            instance = axios_1.default.create(config);
        }
        super(instance);
        this.redirectHeaderKeys = config.redirectHeaderKeys || [
            'x-request-id',
            'x-user',
        ];
        this.config = config;
        this.tracer = tracer;
    }
    request(config) {
        const c = this.injectRedirectHeadersToConfig(config);
        return this.traceWrapper((cc) => super.request(cc).toPromise(), c);
    }
    get(url, config) {
        const c = this.injectRedirectHeadersToConfig(config);
        return this.traceWrapper((cc) => super.get(url, cc).toPromise(), c, url);
    }
    delete(url, config) {
        const c = this.injectRedirectHeadersToConfig(config);
        return this.traceWrapper((cc) => super.delete(url, cc).toPromise(), c, url);
    }
    head(url, config) {
        const c = this.injectRedirectHeadersToConfig(config);
        return this.traceWrapper((cc) => super.head(url, cc).toPromise(), c, url);
    }
    post(url, data, config) {
        const c = this.injectRedirectHeadersToConfig(config);
        return this.traceWrapper((cc) => super.post(url, data, cc).toPromise(), c, url);
    }
    put(url, data, config) {
        const c = this.injectRedirectHeadersToConfig(config);
        return this.traceWrapper((cc) => super.put(url, data, cc).toPromise(), c, url);
    }
    patch(url, data, config) {
        const c = this.injectRedirectHeadersToConfig(config);
        return this.traceWrapper((cc) => super.patch(url, data, cc).toPromise(), c, url);
    }
    traceWrapper(fn, config, url) {
        if (!this.config.withTracing) {
            return rxjs_1.from(fn(config));
        }
        const span = nest_stack_context_1.RequestContext.currentRequest().span;
        const newSpan = this.tracer.startSpan('curl', { childOf: span.context() });
        const headers = Object.assign({}, config.headers);
        newSpan.setTag(opentracing_1.Tags.HTTP_URL, url || config.url);
        newSpan.setTag(opentracing_1.Tags.HTTP_METHOD, (config.method || 'GET').toUpperCase());
        newSpan.setTag(opentracing_1.Tags.SPAN_KIND, opentracing_1.Tags.SPAN_KIND_RPC_CLIENT);
        this.tracer.inject(newSpan, opentracing_1.FORMAT_HTTP_HEADERS, headers);
        config.headers = headers;
        return rxjs_1.from(fn(config)
            .catch((err) => {
            newSpan.setTag(opentracing_1.Tags.ERROR, true);
            throw err;
        })
            .finally(() => {
            newSpan.finish();
        }));
    }
    injectRedirectHeadersToConfig(config) {
        const reqContextHeaders = this.getRequestContextHeaders();
        if (!reqContextHeaders) {
            return config;
        }
        const c = config || {};
        const tracingHeaders = {};
        this.redirectHeaderKeys.forEach((key) => {
            const val = reqContextHeaders[key];
            if (val) {
                tracingHeaders[key] = val;
            }
        });
        if (this.redirectHeaderKeys.includes('x-request-id')) {
            tracingHeaders['x-request-id'] = this.getRequestId();
        }
        const configHeaders = c.headers || {};
        const headers = Object.assign(Object.assign({}, tracingHeaders), configHeaders);
        c.headers = headers;
        return c;
    }
    getRequestContextHeaders() {
        return (nest_stack_context_1.RequestContext.currentRequest() &&
            nest_stack_context_1.RequestContext.currentRequest().headers);
    }
    getRequestId() {
        return nest_stack_context_1.RequestContext.currentRequestId();
    }
};
HttpTracingService = __decorate([
    __param(0, common_1.Inject(http_constants_1.NEST_STACK_HTTP_MODULE_OPTIONS)),
    __param(1, common_1.Inject(http_constants_1.NEST_STACK_AXIOS_INSTANCE_TOKEN)),
    __param(2, common_1.Optional()), __param(2, common_1.Inject(nest_stack_tracing_1.NEST_STACK_TRACER)),
    __metadata("design:paramtypes", [Object, Function, opentracing_1.Tracer])
], HttpTracingService);
exports.HttpTracingService = HttpTracingService;
//# sourceMappingURL=http.service.js.map